//+------------------------------------------------------------------+
//|                                                  FlatTrendV3.mq4 |
//|                                                       Kirk Sloan |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Kirk Sloan"
#property link      "ksfx@kc.rr.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  Red
#property indicator_color2  LimeGreen
#property indicator_color3  Salmon
#property indicator_color4  LightGreen
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  1
#property indicator_width4  1
#property indicator_minimum 0
#property indicator_maximum 1

//---- input parameters
extern string TimeFrame = "current time frame";
extern int    period    = 30;
extern int    price     =  6; // 0 or other = (H+L)/2
                           // 1 = Open
                           // 2 = Close
                           // 3 = High
                           // 4 = Low
                           // 5 = (H+L+C)/3
                           // 6 = (O+C+H+L)/4
                           // 7 = (O+C)/2
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsNotify    = false;
extern bool   alertsEmail     = false;


double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
double Value[];
double Fish[];
double trend1[];
double trend2[];

int    timeFrame;
string indicatorFileName;
bool   returnBars;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,ExtMapBuffer1);  SetIndexStyle(0,DRAW_HISTOGRAM); //modified
   SetIndexBuffer(1,ExtMapBuffer2);  SetIndexStyle(1,DRAW_HISTOGRAM); //modified
   SetIndexBuffer(2,ExtMapBuffer3);  SetIndexStyle(2,DRAW_HISTOGRAM); //modified
   SetIndexBuffer(3,ExtMapBuffer4);  SetIndexStyle(3,DRAW_HISTOGRAM); //modified
   SetIndexBuffer(4,Value);
   SetIndexBuffer(5,Fish);
   SetIndexBuffer(6,trend1);
   SetIndexBuffer(7,trend2);
      timeFrame         = stringToTimeFrame(TimeFrame);
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame=="returnBars"); if (returnBars) return(0);
   IndicatorShortName("Flat Trend V3("+timeFrameToString(timeFrame)+")");  //modified
   return(0);
}
int deinit() { return(0); }

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int countedBars = IndicatorCounted();
      if(countedBars<0) return(-1);
      if(countedBars>0) countedBars--;
         int limit = MathMin(Bars-countedBars,Bars-1);
         if (returnBars) { ExtMapBuffer1[0] = limit+1; return(0); }
         if (timeFrame!=Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));

   //
   //
   //
   //
   //
   
   for (int i = limit; i >= 0; i--)
   {
      ExtMapBuffer1[i]=EMPTY_VALUE;
      ExtMapBuffer2[i]=EMPTY_VALUE;
      ExtMapBuffer3[i]=EMPTY_VALUE;
      ExtMapBuffer4[i]=EMPTY_VALUE;
   
      int y = iBarShift(NULL,timeFrame,Time[i]);
         double PADX = iADX(NULL,timeFrame,14 ,PRICE_CLOSE,MODE_PLUSDI, y);
         double NADX = iADX(NULL,timeFrame,14 ,PRICE_CLOSE,MODE_MINUSDI,y);
         double MaxH = iHigh(NULL,timeFrame,iHighest(NULL,timeFrame,MODE_HIGH,period,y));
         double MinL =  iLow(NULL,timeFrame, iLowest(NULL,timeFrame,MODE_LOW ,period,y));
   
         //
         //
         //
         //
         //
         
         double _price = (iHigh(NULL,timeFrame,i)+iLow(NULL,timeFrame,y))/2;
            if(price == 1) _price = iOpen(NULL,timeFrame,y);
            if(price == 2) _price = iClose(NULL,timeFrame,y);
            if(price == 3) _price = iHigh(NULL,timeFrame,y); 
            if(price == 4) _price = iLow(NULL,timeFrame,y);
            if(price == 5) _price = (iHigh(NULL,timeFrame,y)+iLow(NULL,timeFrame,y)+iClose(NULL,timeFrame,y))/3;
            if(price == 6) _price = (iOpen(NULL,timeFrame,y)+iHigh(NULL,timeFrame,y)+iLow(NULL,timeFrame,y)+iClose(NULL,timeFrame,y))/4; 
            if(price == 7) _price = (iOpen(NULL,timeFrame,y)+iClose(NULL,timeFrame,y))/2; 

         //
         //
         //
         //
         //            
         
         if ((MaxH-MinL)!=0)
               Value[i] = 0.33*2*((_price-MinL)/(MaxH-MinL)-0.5) + 0.67*Value[i+1];     
         else  Value[i] = 0;    
               Value[i] = MathMin(MathMax(Value[i],-0.999),0.999); 
               Fish[i]  = 0.5*MathLog((1+Value[i])/(1-Value[i])) -0.5*Fish[i+1] -0.5*Fish[i+2];
         trend1[i] = trend1[i+1];
         trend2[i] = trend2[i+1]; 
         if (Fish[i]>=0)  trend1[i] = 1;
         if (Fish[i]<=0)  trend1[i] =-1;
         if (PADX > NADX) trend2[i] = 1;
         if (NADX > PADX) trend2[i] =-1;
         if (trend1[i] == 1 && trend2[i] == 1)   ExtMapBuffer2[i] = 1;
         if (trend1[i] ==-1 && trend2[i] ==-1)   ExtMapBuffer1[i] = 1;
         if (trend1[i] == 1 && trend2[i] ==-1)   ExtMapBuffer4[i] = 1;
         if (trend1[i] ==-1 && trend2[i] == 1)   ExtMapBuffer3[i] = 1;
         
   }
   manageAlerts();
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs) {
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf) {
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str) {
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--) {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      
      //
      //
      //
      //
      //
      
      
         if (trend1[whichBar] != trend1[whichBar+1] || trend2[whichBar] != trend2[whichBar+1])
         {
            if (trend1[whichBar] == 1 && trend2[whichBar] == 1) doAlert(whichBar,"strong up");
            if (trend1[whichBar] ==-1 && trend2[whichBar] ==-1) doAlert(whichBar,"strong down");
            if (trend1[whichBar] == 1 && trend2[whichBar] ==-1) doAlert(whichBar,"weak up");
            if (trend1[whichBar] ==-1 && trend2[whichBar] == 1) doAlert(whichBar,"weak down");
         }                
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Flat Trend ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Flat Trend"),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
} 
 
   